#include <malloc.h>
#include <string.h>
#include <stdio.h>
#include <mpi.h>

#define NUM_ITERATIONS 100000

double ptp_latency_test(int n1, int n2, int size);
double bcast_latency_test(int n1, int n2, int size);

main(int argc, char *argv[])
{
	int rank, size;
	int i,j;
	MPI_Status s;
	double latency;

	MPI_Init(&argc, &argv);
	MPI_Comm_rank(MPI_COMM_WORLD, &rank);
	MPI_Comm_size(MPI_COMM_WORLD, &size);
	MPI_Barrier(MPI_COMM_WORLD);

	if(rank==0) printf("Number Processors: %d\n", size);
	for(i=0;i<size;i++) {
		for(j=0;j<size;j++) {
			latency=ptp_latency_test(i,j,0); 
			if(rank==i){
				printf("%d %d %.3e\t",i,j,latency); 
				fflush(stdout);
			}
		}
	}

	MPI_Barrier(MPI_COMM_WORLD);
	MPI_Finalize();
	exit(0);
}


double ptp_latency_test(int n1, int n2, int msgsize) {
	int i, rank;
	double t1, t2, t, latency=-1.0;
	char *buf;
	MPI_Status s;

	buf =  (char *)malloc(sizeof(char)*(msgsize+1));
 
	MPI_Comm_rank(MPI_COMM_WORLD, &rank);
	MPI_Barrier(MPI_COMM_WORLD);
	if ((rank==n1) || (rank==n2)) { 
		t1 = MPI_Wtime();
		for (i=0; i<NUM_ITERATIONS; i++) {
			if (rank == n1) {
				MPI_Send(&buf, msgsize, MPI_BYTE, n2, 0, MPI_COMM_WORLD);
				MPI_Recv(&buf, msgsize, MPI_BYTE, n2, 0, MPI_COMM_WORLD, &s);
			} else {
				MPI_Recv(&buf, msgsize, MPI_BYTE, n1, 0, MPI_COMM_WORLD, &s);
				MPI_Send(&buf, msgsize, MPI_BYTE, n1, 0, MPI_COMM_WORLD);
			}
		}
		t2 = MPI_Wtime();
	}
	MPI_Barrier(MPI_COMM_WORLD);

	t = t2 - t1;
	latency = 1.0e6 * t/((double)NUM_ITERATIONS * 2);	//x2 (t == round trip)
	return latency;
}
